#!/bin/bash
set -e

# Files created by Elasticsearch should always be group writable too
umask 0002


run_as_other_user_if_needed() {
  if [[ "$(id -u)" == "0" ]]; then
    # If running as root, drop to specified UID and run command
    exec chroot --userspec=1000 / "${@}"
  else
    # Either we are running in Openshift with random uid and are a member of the root group
    # or with a custom --user
    exec "${@}"
  fi
}

# Allow user specify custom CMD, maybe bin/elasticsearch itself
# for example to directly specify `-E` style parameters for elasticsearch on k8s
# or simply to run /bin/bash to check the image
if [[ "$1" != "eswrapper" ]]; then
  if [[ "$(id -u)" == "0" && $(basename "$1") == "elasticsearch" ]]; then
    # centos:7 chroot doesn't have the `--skip-chdir` option and
    # changes our CWD.
    # Rewrite CMD args to replace $1 with `elasticsearch` explicitly,
    # so that we are backwards compatible with the docs
    # from the previous Elasticsearch versions<6
    # and configuration option D:
    # https://www.elastic.co/guide/en/elasticsearch/reference/5.6/docker.html#_d_override_the_image_8217_s_default_ulink_url_https_docs_docker_com_engine_reference_run_cmd_default_command_or_options_cmd_ulink
    # Without this, user could specify `elasticsearch -E x.y=z` but
    # `bin/elasticsearch -E x.y=z` would not work.
    set -- "elasticsearch" "${@:2}"
    # Use chroot to switch to UID 1000
    exec chroot --userspec=1000 / "$@"
  else
    # User probably wants to run something else, like /bin/bash, with another uid forced (Openshift?)
    exec "$@"
  fi
fi

# Parse Docker env vars to customize Elasticsearch
#
# e.g. Setting the env var cluster.name=testcluster
#
# will cause Elasticsearch to be invoked with -Ecluster.name=testcluster
#
# see https://www.elastic.co/guide/en/elasticsearch/reference/current/settings.html#_setting_default_settings

declare -a es_opts

while IFS='=' read -r envvar_key envvar_value
do
  # Elasticsearch settings need to have at least two dot separated lowercase
  # words, e.g. `cluster.name`, except for `processors` which we handle
  # specially
  if [[ "$envvar_key" =~ ^[a-z0-9_]+\.[a-z0-9_]+ || "$envvar_key" == "processors" ]]; then
    if [[ ! -z $envvar_value ]]; then
      es_opt="-E${envvar_key}=${envvar_value}"
      es_opts+=("${es_opt}")
    fi
  fi
done < <(env)

# The virtual file /proc/self/cgroup should list the current cgroup
# membership. For each hierarchy, you can follow the cgroup path from
# this file to the cgroup filesystem (usually /sys/fs/cgroup/) and
# introspect the statistics for the cgroup for the given
# hierarchy. Alas, Docker breaks this by mounting the container
# statistics at the root while leaving the cgroup paths as the actual
# paths. Therefore, Elasticsearch provides a mechanism to override
# reading the cgroup path from /proc/self/cgroup and instead uses the
# cgroup path defined the JVM system property
# es.cgroups.hierarchy.override. Therefore, we set this value here so
# that cgroup statistics are available for the container this process
# will run in.
export ES_JAVA_OPTS="-Des.cgroups.hierarchy.override=/ $ES_JAVA_OPTS"

if [[ -z "$ELASTICSEARCH_KEYSTORE" ]]; then
  ELASTICSEARCH_KEYSTORE=9d5209e7f77ee847189166
fi

if [[ -f bin/elasticsearch-users ]]; then
  # Check for the ELASTICSEARCH_PASSWORD environment variable to set the
  # bootstrap password for Security.
  #
  # This is only required for the first node in a cluster with Security
  # enabled, but we have no way of knowing which node we are yet. We'll just
  # honor the variable if it's present.

  if [[ -n "$ELASTICSEARCH_PASSWORD" ]]; then
    if [[ ! -f /usr/share/elasticsearch/config/elasticsearch.keystore ]]; then
     if [[ -f /usr/share/elasticsearch/config/certs/elasticsearch.keystore ]]; then
       cp -p /usr/share/elasticsearch/config/certs/elasticsearch.keystore /usr/share/elasticsearch/config/elasticsearch.keystore
     else
       (run_as_other_user_if_needed elasticsearch-keystore create)
     fi
    fi
    if ! (run_as_other_user_if_needed elasticsearch-keystore has-passwd --silent) ; then
      # keystore is unencrypted
      if ! (run_as_other_user_if_needed elasticsearch-keystore list | grep -q '^bootstrap.password$'); then
        (run_as_other_user_if_needed echo "$ELASTICSEARCH_PASSWORD" | elasticsearch-keystore add -x 'bootstrap.password')
      fi
    else
      # keystore requires password
      if ! (run_as_other_user_if_needed echo "$ELASTICSEARCH_KEYSTORE" \
          | elasticsearch-keystore list | grep -q '^bootstrap.password$') ; then
        COMMANDS="$(printf "%s\n%s" "$ELASTICSEARCH_KEYSTORE" "$ELASTICSEARCH_PASSWORD")"
        (run_as_other_user_if_needed echo "$COMMANDS" | elasticsearch-keystore add -x 'bootstrap.password')
      fi
    fi
  fi
fi

if [ ! -f /usr/share/elasticsearch/config/certs/elastic-ca.crt ] || [ ! -f /usr/share/elasticsearch/config/certs/elastic-certificates.p12 ] ; then
  cp -fv /opt/tmp-es/certs/* /usr/share/elasticsearch/config/certs/
fi

# Additional layer of security to ensure that all communications to and from your cluster are encrypted.
sed -i "s/NODE_ID/${NODE_ID}/g" "/usr/share/elasticsearch/config/instances.yml"

if [ ! -f /usr/share/elasticsearch/config/certs/http.zip ]; then
  elasticsearch-certutil cert --silent --in "/usr/share/elasticsearch/config/instances.yml" --out "/usr/share/elasticsearch/config/certs/http.zip" --days 3065 --ca-pass  "${ELASTICSEARCH_KEYSTORE}" --pass "${ELASTICSEARCH_KEYSTORE}" --ca  "/usr/share/elasticsearch/config/certs/elastic-stack-ca.p12"
fi

if [ ! -f /usr/share/elasticsearch/config/certs/http.p12 ]; then
  unzip /usr/share/elasticsearch/config/certs/http.zip -d /usr/share/elasticsearch/config/certs/
  cp -f /usr/share/elasticsearch/config/certs/http/http.p12 /usr/share/elasticsearch/config/certs/http.p12
  /usr/share/elasticsearch/jdk/bin/keytool -importkeystore -srcstorepass "${ELASTICSEARCH_KEYSTORE}" -deststorepass "${ELASTICSEARCH_KEYSTORE}" -destkeystore "/usr/share/elasticsearch/config/certs/http.p12" -srckeystore "/usr/share/elasticsearch/config/certs/elastic-stack-ca.p12" -srcstoretype PKCS12 --noprompt
fi

if [ ! -f /usr/share/elasticsearch/config/certs/http.pem ]; then
  openssl pkcs12 -nodes -in "/usr/share/elasticsearch/config/certs/http.p12" -out "/usr/share/elasticsearch/config/certs/http.pem" -password pass:${ELASTICSEARCH_KEYSTORE}
fi

if [ ! -f /usr/share/elasticsearch/config/certs/http_ca.crt ]; then
  openssl x509 -outform der -in "/usr/share/elasticsearch/config/certs/http.pem" -out "/usr/share/elasticsearch/config/certs/http_ca.crt"
  # Add keystore for http and transport layers
  (run_as_other_user_if_needed echo "$ELASTICSEARCH_KEYSTORE" | elasticsearch-keystore add 'xpack.security.transport.ssl.keystore.secure_password' -x --force)
  (run_as_other_user_if_needed echo "$ELASTICSEARCH_KEYSTORE" | elasticsearch-keystore add 'xpack.security.transport.ssl.truststore.secure_password' -x --force)
  (run_as_other_user_if_needed echo "$ELASTICSEARCH_KEYSTORE" | elasticsearch-keystore add 'xpack.security.http.ssl.keystore.secure_password' -x --force)
fi

# Create elasticsearch user
if ! (run_as_other_user_if_needed elasticsearch-users list | grep -q "$ELASTICSEARCH_USERNAME"); then
    (run_as_other_user_if_needed elasticsearch-users useradd ${ELASTICSEARCH_USERNAME} -p ${ELASTICSEARCH_PASSWORD} -r superuser)
fi

if [[ "$(id -u)" == "0" ]]; then
  # If requested and running as root, mutate the ownership of bind-mounts
  if [[ -n "$TAKE_FILE_OWNERSHIP" ]]; then
    chown -R 1000:0 /usr/share/elasticsearch/{data,logs}
  fi
fi

if [[ -n "$ES_LOG_STYLE" ]]; then
  case "$ES_LOG_STYLE" in
    console)
      # This is the default. Nothing to do.
      ;;
    file)
      # Overwrite the default config with the stack config. Do this as a
      # copy, not a move, in case the container is restarted.
      cp -f /usr/share/elasticsearch/config/log4j2.file.properties /usr/share/elasticsearch/config/log4j2.properties
      ;;
    *)
      echo "ERROR: ES_LOG_STYLE set to [$ES_LOG_STYLE]. Expected [console] or [file]" >&2
      exit 1 ;;
  esac
fi

if [[ -f /usr/share/elasticsearch/config/elasticsearch.keystore ]]; then
  cp -p /usr/share/elasticsearch/config/elasticsearch.keystore /usr/share/elasticsearch/config/certs/
fi
run_as_other_user_if_needed /usr/share/elasticsearch/bin/elasticsearch "${es_opts[@]}" <<<"$ELASTICSEARCH_KEYSTORE"